﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization;
using Newtonsoft.Json.Linq;
using System.Reflection;
using System.Collections;
using System.Dynamic;

namespace InfoAPILib
{
    /// <summary>
    /// InfoDatabaseAttachment sadrži listu tabela koje su uključene u API poruku.
    /// </summary>
    public class InfoAPIDatabaseAttachment
    {
        /// <summary>
        /// Broj tabela u listi
        /// </summary>
        ///[DataMember(EmitDefaultValue = false)]
        public int count { get; set; }

        /// Naziv klase. U API porukama je ovo uglavnom informativnog karaktera, a dobro dođe 
        /// kada se analiziraju složene strukture
        /// </summary>        
        [DataMember(EmitDefaultValue = false)]
        public string _class_name { get; set; }

        /// <summary>
        /// Lista tabela
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public List<DatabaseAttachmentTable> items { get; set; }

        /// <summary>
        /// Konstruktor
        /// </summary>
        public InfoAPIDatabaseAttachment()
        {
            this.items = new List<DatabaseAttachmentTable>();
        }

        /// <summary>
        /// Učitava tabele iz liste referenci na tabele. 
        /// 
        /// Korisnik biblioteke, prilikom pripreme 
        /// upita, svoje tabele povezuje samo kao reference, a tek kada se započne postupak 
        /// serijalizacije, na osnovu tih referenci se učitavaju podaci da bi bili serijalizovani
        /// i uključeni u upit.
        /// </summary>
        /// <param name="pTableReferences"></param>
        public void LoadFromTableReferences(InfoAPITableReferences pTableReferences)
        {
            if (pTableReferences.count > 0)
            {
                foreach (InfoAPITableReference lTableReference in pTableReferences.items)
                {
                    IEnumerable<object> lTableData = lTableReference.GetData();
                    if ((lTableData != null))
                    {
                        this.count++;
                        DatabaseAttachmentTable lDatabaseAttachmentTable = new DatabaseAttachmentTable();
                        lDatabaseAttachmentTable.LoadFromTableReference(lTableReference);
                        this.items.Add(lDatabaseAttachmentTable);
                    }

                }

            }

        }

    }

    /// <summary>
    /// Sadrži opis jedne tabele
    /// </summary>
    public class DatabaseAttachmentTable
    {
        /// <summary>
        /// Lista polja/kolona u tabeli.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public DatabaseAttachmentFields fields { get; set; }

        /// <summary>
        /// Lista slogova tabele
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public DatabaseAttachmentRows rows { get; set; }

        /// <summary>
        /// Metod koji daje informaciju da li redovi tabele teba da se serijalizuju. 
        /// Najčešći slučaj je da je tabela prazna, i tada se tabela šalje u poruci ali 
        /// se ne serijalizuju slogovi jer ih nema.
        /// </summary>
        /// <returns></returns>
        public bool ShouldSerializerows()
        {
            return this.rows.items.Count > 0;
        }

        /// <summary>
        /// Naziv tabele. Ovaj naziv je po pravilu definisan u deklaraciji API funkcije i 
        /// po njemu API funkcija nalazi podatke u odgovarajućoj tabeli.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string table_name { get; set; }

        /// <summary>
        /// Konstruktor
        /// </summary>
        public DatabaseAttachmentTable()
        {
            this.fields = new DatabaseAttachmentFields();
            this.rows = new DatabaseAttachmentRows();
        }

        /// <summary>
        /// Prepiše podatke iz slogova u listi u neku spoljnu strukturu.
        /// Ovo se upoterbljava kada se primaju tabelarni podaci kao rezultat poziva API funkcije. Radi
        /// uštede u resursima, sadržaj tabele se ne deserijalizuje automatski, nego tek kada je 
        /// potreban, pomoću ovog metoda.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="pDestinationTable"></param>
        public void Populate<T> (IList<T> pDestinationTable)
        {
            Type mListType = pDestinationTable.GetType();

            Type mGenericType;

            // provera da li je ispravan tip liste
            if (mListType.IsGenericType)
            {
                mGenericType = mListType.GetGenericArguments()[0];
            }
            else
            {
                mGenericType = mListType.BaseType.GetGenericArguments()[0];
            }

            if (mGenericType.IsGenericType && mGenericType.GetGenericTypeDefinition() == typeof(List<>))
            {
                throw new Exception("Not proper table structure: " + mListType + ". List can not contain List");
            }

            foreach (object lRow in this.rows.items)
            {
                //JRaw mRow = (JRaw)mRowObj;
                string lRowStr = ((JObject)lRow).ToString();

                object mDestinationRow = Activator.CreateInstance(mGenericType, null);

                //mDestinationRow = Newtonsoft.Json.JsonConvert.DeserializeObject(mRow.Value.ToString(), mGenericType);
                mDestinationRow = Newtonsoft.Json.JsonConvert.DeserializeObject(lRowStr, mGenericType);

                pDestinationTable.Add((T)mDestinationRow);
            }
        }

        /// <summary>
        /// Učita tabelu iz reference
        /// </summary>
        /// <param name="pTableReference"></param>
        public void LoadFromTableReference(InfoAPITableReference pTableReference)
        {
            IEnumerable<object> lTableData = pTableReference.GetData();

            this.table_name = pTableReference.table_name;
            this.CreateFields(pTableReference);

            if ((lTableData != null) && (lTableData.Count() > 0))
            {
                this.rows.items.AddRange (pTableReference.data);
            }


        }

        /// <summary>
        /// Kreira opis polja na osnovu strukture podatka
        /// Preslikava metod DatabaseAttachmentFields.CreateFields()
        /// </summary>
        /// <param name="pData"></param>
        private void CreateFields (InfoAPITableReference pTableReference)
        {
            this.fields.CreateFields(pTableReference);
        }

    }



    /// <summary>
    /// Lista slogova tabele
    /// </summary>
    public class DatabaseAttachmentRows
    {
        /// <summary>
        /// Broj slogova
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public int count {
            get { return this.items.Count; }
            private set { }
        }

        /// <summary>
        /// Lista slogova
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public List<object> items { get; set; }

        /// <summary>
        /// obsolete
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string keysort { get; set; }

        /// <summary>
        /// Konstruktor
        /// </summary>
        public DatabaseAttachmentRows()
        {
            this.items = new List<object>();
        }

    }

    /// <summary>
    /// Lista polja/kolona u tabeli, opis strukture tabele.
    /// 
    /// Obično je struktura tabele unaped definisana dekalracijom API funkcije. Međutim, može se desiti
    /// da API funkcia en zna unapred strukturu tabele koju dobija i ova lista zbog toga sadrži opis te strukture.
    /// 
    /// Čak i ako je strukturaunaped poznata, neke platforme ne mogu da strukturu tabele pročitaju 
    /// iz samih podataka pa im je potrebno da je dobiju posebno opisanu. Koristi se i kada se ne 
    /// zna unapred struktura tabele.
    /// </summary>
    public class DatabaseAttachmentFields
    {
        /// <summary>
        /// Broj polja/kolina
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public int count
        {
            get { return this.items.Count; }
            private set { }
        }

        /// <summary>
        /// lista polja
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public List<DatabaseAttachmentField> items { get; set; }

        /// <summary>
        /// obsolete
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string keysort { get; set; }

        /// <summary>
        /// Konstruktor
        /// </summary>
        public DatabaseAttachmentFields()
        {
            this.items = new List<DatabaseAttachmentField>();
        }

        /// <summary>
        // Mapira .NET nazive tipova u InfoAPI tipove
        /// </summary>
        /// <param name="pType">Naziv .NET tipa kolone</param>
        /// <returns>Oznaka tipa kolone prema InfoAPI specifikaciji</returns>
        public string MapPropertyType (Type pType)
        {
            string lResult = "";

            switch (pType.Name)
            {
                case "String":
                case "Char":
                    lResult = "C";
                    break;

                case "Int32":
                case "UInt32":
                case "Int64":
                case "UInt64":
                case "Int16":
                case "UInt16":
                case "Byte":
                case "SByte":
                    lResult = "I";
                    break;

                case "Decimal":
                case "Double":
                case "Single":
                case "Float":
                    lResult = "N";
                    break;

                case "DateTime":
                    lResult = "D";
                    break;

                case "Boolean":
                    lResult = "D";
                    break;


            }
            return lResult;

        }


        /// <summary>
        /// Dodaj opis jednog polja u listu
        /// </summary>
        /// <param name="pName">Naziv polja</param>
        /// <param name="pType">Tip polja prema InfoAPI specifikaciji</param>
        public void AddField(string pName, string pType)
        {
            DatabaseAttachmentField lField = new DatabaseAttachmentField(pName, pType);
            this.items.Add(lField);
        }

        /// <summary>
        /// Dodaj opis jednog polja u listu
        /// </summary>
        /// <param name="pName">Naziv polja</param>
        /// <param name="pType">Tip polja prema .NET specifikaciji</param>
        public void AddField(string pName, Type pType)
        {
            string lType = this.MapPropertyType(pType);
            this.AddField(pName, lType);
        }

        /// <summary>
        /// Dodaj opis jednog polja u listu prema PropertyInfo
        /// </summary>
        /// <param name="pProperty">PropertyInfo opis polja </param>
        public void AddField(PropertyInfo pProperty)
        {
            
            this.AddField(pProperty.Name, pProperty.PropertyType);
        }

        /// <summary>
        /// Kreira opis svih polja na osnovu strukture podatka
        /// </summary>
        /// <param name="pData"></param>
        public void CreateFields(InfoAPITableReference pTableReference)
        {
            IEnumerable<object> mData = pTableReference.GetData();

            this.items.Clear();

            //Type lClassTypeA = mData.GetType().BaseType.GetGenericArguments()[0];
            Type lClassType = pTableReference.GetDataType();

            bool hasDataContractAttribute = lClassType.GetCustomAttributes(typeof(DataContractAttribute), true).Any();

            PropertyInfo[] lProperties = lClassType.GetProperties(BindingFlags.Instance | BindingFlags.Public);
            foreach (PropertyInfo lProperty in lProperties)
            {

                bool hasDataMemberAttribute = lProperty.GetCustomAttributes(typeof(DataMemberAttribute), true).Any();
                bool hasIgnoreDataMemberAttribute = lProperty.GetCustomAttributes(typeof(IgnoreDataMemberAttribute), true).Any();

                if ((hasDataContractAttribute && hasDataMemberAttribute) || (!hasDataContractAttribute && !hasIgnoreDataMemberAttribute) ) { 
                    this.AddField(lProperty);
                }
            }

            if (this.items.Count == 0)
            {
                if (mData is IEnumerable)
                //if (pData is IDynamicMetaObjectProvider)
                    
                {
                    if (mData.Count() > 0)
                    {
                        dynamic lRow = mData.First();
                        IDictionary<string, object> lPropertyValues = lRow;
                        foreach (var lDynamicProperty in lPropertyValues)
                        {
                            this.AddField(lDynamicProperty.Key, lDynamicProperty.Value.GetType());
                        }
                    }
                }
            }

        }


    }

    /// <summary>
    /// Opis jednog polja/kolone
    /// </summary>
    public class DatabaseAttachmentField
    {
        /// <summary>
        /// Naziv polja
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string name { get; set; }

        /// <summary>
        /// Tip polja po InfoAPI specifikaciji
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string type { get; set; }

        /// <summary>
        /// Konstruktor
        /// </summary>
        public DatabaseAttachmentField() { }

        /// <summary>
        /// Konstruktor koji odmah kreira opis prema ulaznim parametrima
        /// </summary>
        /// <param name="pName">Naziv polja</param>
        /// <param name="pType">Tip polja prema InfoAPI specifikaciji</param>
        public DatabaseAttachmentField(string pName, string pType)
        {
            this.name = pName;
            this.type = pType;
        }

    }


}
