﻿using System;
using System.Text;
using System.Security.Cryptography;
using System.Runtime.Serialization;

namespace InfoAPILib
{
    /// <summary>
    /// Omotnica API poruke. Ovo je zajednička bazna klasa za InfoAPIRequest i InfoAPIResponse
    /// </summary>
    public class InfoAPIEnvelope : InfoAPIBase
    {
        private string _req_id = "";

        /// <summary>
        /// req_id (char)
        /// Идентификатор упита.Овај податак генерише клијент а сервер га враћа
        /// у одговору.Служи да клијент може да по потреби повеже одговоре са
        /// упитима и за рачунање контролног кода за проверу целевитости
        /// послатог упита.Клијент треба да употребљава различиту вредност за
        ///сваки упит који шаље.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string req_id
        {
            get
            {
                if (this._req_id == "")
                {
                    this.ResetReqId();
                }
                return this._req_id;
            }
            set
            {
                this._req_id = value;
            }
        }

        /// <summary>
        /// Restuje req_id u InfoAPIEnvelope. Mora biti pozvan svaki put pre slanja upita API serveru, da bi se req_id promenio.
        /// </summary>
        /// <returns>String vrednost podešenu za req_id</returns>
        public string ResetReqId()
        {
            this._req_id = Guid.NewGuid().ToString("N");
            return this._req_id;
        }

        /// <summary>
        /// session_id (char) 
        /// Идентификатор сесије. Овај идентификатор генерише сервер када
        /// клијент први пут позове АПИ и враћа га клијенту. Клијент у сваком
        /// следећем упиту у оквиру исте сесије шаље идентификатор сесије.
        /// Ово омогућава да се одређени подаци или стања могу преносити
        /// између више упита са истог клијента ка АПИ-ју.
        /// Сесија се прекида одјавом или неактивношћу клијента. Подразумевано,
        /// некативност дужа од 10 минута прекида сесију, а то се може
        /// подешавати на АПИ северу.
        /// Ако клијент не зна идентификатор сесије, односно тек започиње сесију,
        /// шаље ово поље празно.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string session_id { get; set; }

        /// <summary>
        /// instance_id(char) 
        /// Идентификатор инстанце у оквиру сесије.Овај идентификатор сервер
        /// генерише ако није добио идентификатор од клијента.Клијент по
        /// правилу у сваком наредном упиту шаље идентификатор инстанце који
        /// је добио од сервера осим ако не жели да започне нову инстанцу.
        /// Клијент може да покрене више инстанци у оквиру исте сесије. Све
        /// инстанце се затварају са затварањем сесије.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string instance_id { get; set; }

        /// <summary>
        /// encoding(char) 
        /// Начин на који су кодирани подаци у пољу data. Сервер на основу овог податка зна како да декодира податке.
        /// Подразумевано се подаци не кодирају и ово поље остаје празно. Енкодирање података није још увек имплементирано.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string encoding { get; set; }

        /// <summary>
        /// data (char) 
        /// Параметри који се шаљу АПИ функцији.Ово поље садржи угњеждену
        /// ЈСОН структуру TAPIParams. Та структура је променљива зависно од
        /// функције која се позива. 
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string data { get; set; }

        /// <summary>
        /// api_version (char) 
        /// Ознака верзије АПИ-ја који клијент разуме
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string api_version { get; protected set; }

        /// <summary>
        /// poslovna_godina (char) 
        /// Ознака пословне године. Употребљава се ако клијент треба да наведе
        /// АПИ-ју коју пословну годину жели да употреби. Ретко се употребљава и
        /// подразумевано је поље празно, што значи да ће АПИ употребљавати
        /// податке из текуће пословне године
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string poslovna_godina { get; set; }

        /// <summary>
        /// job (char) 
        /// Ознака JOB-а. Употребљава се ако клијент треба да наведе у ком JOB-у
        /// жели да покрене АПИ функцију.Користи се само изузетно и
        /// подразумевано поље остаје празно.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string job { get; set; }

        /// <summary>
        /// pozicija(char) 
        /// Ознака позиције.Употребљава се ако клијент треба да наведе на којој
        /// позицији жели да покрене АПИ функцију.Користи се само изузетно и
        /// подразумевано поље остаје празно.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string pozicija { get; set; }

        /// <summary>
        /// checksum(char) 
        /// Контролни код поруке.Намена му је да се преко њега може проверити
        /// интегритет послатог упита.
        /// </summary>
        [DataMember(EmitDefaultValue = false)]
        public string checksum { get; protected set; }


        public InfoAPIEnvelope()
        {
            
        }

        /// <summary>
        /// Izračuna čeksum na osnovu data, req_id i privatnog ključa
        /// </summary>
        /// <param name="pData">sadržaj data svojstva</param>
        /// <param name="pRequestId">sadržaj req_id</param>
        /// <param name="pPrivatekey"></param>
        /// <returns>izračunati čeksum kao string</returns>
        public string GetChecksum (string pData, string pRequestId, string pPrivatekey)
        {
            MD5 md5 = MD5.Create();
            byte[] inputBytes = Encoding.ASCII.GetBytes(pData + pRequestId + pPrivatekey);
            byte[] hash = md5.ComputeHash(inputBytes);

            string mResult = BitConverter.ToString(hash).Replace("-", "").ToLower();

            return mResult;
        }
    }
}
